@@ -34,7 +34,7 @@ PyObject *igraphmodule__disjoint_union(PyObject *self,
 {
   static char* kwlist[] = { "graphs", NULL };
   PyObject *it, *graphs;
-  long int no_of_graphs;
+  Py_ssize_t no_of_graphs;
   igraph_vector_ptr_t gs;
   PyObject *result;
   PyTypeObject *result_type;
@@ -62,13 +62,12 @@ PyObject *igraphmodule__disjoint_union(PyObject *self,
     return NULL;
   }
   Py_DECREF(it);
-  no_of_graphs = (long int) igraph_vector_ptr_size(&gs);
+  no_of_graphs = igraph_vector_ptr_size(&gs);
 
   /* Create disjoint union */
   if (igraph_disjoint_union_many(&g, &gs)) {
     igraph_vector_ptr_destroy(&gs);
-    igraphmodule_handle_igraph_error();
-    return NULL;
+    return igraphmodule_handle_igraph_error();
   }
 
   igraph_vector_ptr_destroy(&gs);
@@ -80,8 +79,7 @@ PyObject *igraphmodule__disjoint_union(PyObject *self,
     result = igraphmodule_Graph_subclass_from_igraph_t(
         result_type,
         &g);
-  }
-  else {
+  } else {
     result = igraphmodule_Graph_from_igraph_t(&g);
   }
 
@@ -98,7 +96,7 @@ PyObject *igraphmodule__union(PyObject *self,
   static char* kwlist[] = { "graphs", "edgemaps", NULL };
   PyObject *it, *em_list = 0, *graphs, *with_edgemaps_o;
   int with_edgemaps = 0;
-  long int no_of_graphs;
+  Py_ssize_t i, j, no_of_graphs;
   igraph_vector_ptr_t gs;
   igraphmodule_GraphObject *o;
   PyObject *result;
@@ -130,13 +128,13 @@ PyObject *igraphmodule__union(PyObject *self,
     return NULL;
   }
   Py_DECREF(it);
-  no_of_graphs = (long int) igraph_vector_ptr_size(&gs);
+
+  no_of_graphs = igraph_vector_ptr_size(&gs);
 
   if (with_edgemaps) {
     /* prepare edgemaps */
-    igraph_vector_ptr_t edgemaps;
-
-    if (igraph_vector_ptr_init(&edgemaps, 0)) {
+    igraph_vector_int_list_t edgemaps;
+    if (igraph_vector_int_list_init(&edgemaps, 0)) {
       igraph_vector_ptr_destroy(&gs);
       return igraphmodule_handle_igraph_error();
     }
@@ -144,30 +142,41 @@ PyObject *igraphmodule__union(PyObject *self,
     /* Create union */
     if (igraph_union_many(&g, &gs, &edgemaps)) {
       igraph_vector_ptr_destroy(&gs);
-      igraph_vector_ptr_destroy(&edgemaps);
+      igraph_vector_int_list_destroy(&edgemaps);
       return igraphmodule_handle_igraph_error();
     }
 
     /* extract edgemaps */
-    long int i;
-    em_list = PyList_New((Py_ssize_t) no_of_graphs);
+    em_list = PyList_New(no_of_graphs);
     for (i = 0; i < no_of_graphs; i++) {
-      long int j;
-      long int no_of_edges = (long int) igraph_ecount(VECTOR(gs)[i]);
-      igraph_vector_t *map = VECTOR(edgemaps)[i];
-      PyObject *emi = PyList_New((Py_ssize_t) no_of_edges);
-      for (j = 0; j < no_of_edges; j++) {
-        PyObject *dest = PyLong_FromLong(VECTOR(*map)[j]);
-        PyList_SET_ITEM(emi, (Py_ssize_t) j, dest);
-        /* reference to 'dest' stolen by PyList_SET_ITEM */
+      Py_ssize_t no_of_edges = igraph_ecount(VECTOR(gs)[i]);
+      igraph_vector_int_t *map = igraph_vector_int_list_get_ptr(&edgemaps, i);
+      PyObject *emi = PyList_New(no_of_edges);
+      if (emi) {
+        for (j = 0; j < no_of_edges; j++) {
+          PyObject *dest = igraphmodule_integer_t_to_PyObject(VECTOR(*map)[j]);
+          if (!dest || PyList_SetItem(emi, j, dest)) {
+            igraph_vector_ptr_destroy(&gs);
+            igraph_vector_int_list_destroy(&edgemaps);
+            Py_XDECREF(dest);
+            Py_DECREF(emi);
+            Py_DECREF(em_list);
+            return NULL;
+          }
+          /* reference to 'dest' stolen by PyList_SetItem */
+        }
       }
-      PyList_SET_ITEM(em_list, (Py_ssize_t) i, emi);
-      /* reference to 'emi' stolen by PyList_SET_ITEM */
-      igraph_vector_destroy(map);
+      if (!emi || PyList_SetItem(em_list, i, emi)) {
+        igraph_vector_ptr_destroy(&gs);
+        igraph_vector_int_list_destroy(&edgemaps);
+        Py_XDECREF(emi);
+        Py_DECREF(em_list);
+        return NULL;
+      }
+      /* reference to 'emi' stolen by PyList_SetItem */
     }
-    igraph_vector_ptr_destroy_all(&edgemaps);
-  }
-  else {
+    igraph_vector_int_list_destroy(&edgemaps);
+  } else {
     /* Create union */
     if (igraph_union_many(&g, &gs, /* edgemaps */ 0)) {
       igraph_vector_ptr_destroy(&gs);
@@ -184,8 +193,7 @@ PyObject *igraphmodule__union(PyObject *self,
     o = (igraphmodule_GraphObject*) igraphmodule_Graph_subclass_from_igraph_t(
         result_type,
         &g);
-  }
-  else {
+  } else {
     o = (igraphmodule_GraphObject*) igraphmodule_Graph_from_igraph_t(&g);
   }
 
@@ -196,8 +204,7 @@ PyObject *igraphmodule__union(PyObject *self,
     Py_DECREF(o);
     PyDict_SetItemString(result, "edgemaps", em_list);
     Py_DECREF(em_list);
-  }
-  else {
+  } else {
     result = (PyObject *) o;
   }
 
@@ -213,7 +220,7 @@ PyObject *igraphmodule__intersection(PyObject *self,
   static char* kwlist[] = { "graphs", "edgemaps", NULL };
   PyObject *it, *em_list = 0, *graphs, *with_edgemaps_o;
   int with_edgemaps = 0;
-  long int no_of_graphs;
+  Py_ssize_t i, j, no_of_graphs;
   igraph_vector_ptr_t gs;
   igraphmodule_GraphObject *o;
   PyObject *result;
@@ -245,12 +252,12 @@ PyObject *igraphmodule__intersection(PyObject *self,
     return NULL;
   }
   Py_DECREF(it);
-  no_of_graphs = (long int) igraph_vector_ptr_size(&gs);
+  no_of_graphs = igraph_vector_ptr_size(&gs);
 
   if (with_edgemaps) {
     /* prepare edgemaps */
-    igraph_vector_ptr_t edgemaps;
-    if (igraph_vector_ptr_init(&edgemaps, 0)) {
+    igraph_vector_int_list_t edgemaps;
+    if (igraph_vector_int_list_init(&edgemaps, 0)) {
       igraph_vector_ptr_destroy(&gs);
       return igraphmodule_handle_igraph_error();
     }
@@ -258,35 +265,44 @@ PyObject *igraphmodule__intersection(PyObject *self,
     /* Create intersection */
     if (igraph_intersection_many(&g, &gs, &edgemaps)) {
       igraph_vector_ptr_destroy(&gs);
-      igraph_vector_ptr_destroy(&edgemaps);
+      igraph_vector_int_list_destroy(&edgemaps);
       return igraphmodule_handle_igraph_error();
     }
 
-    long int i;
     em_list = PyList_New((Py_ssize_t) no_of_graphs);
     for (i = 0; i < no_of_graphs; i++) {
-      long int j;
-      long int no_of_edges = (long int) igraph_ecount(VECTOR(gs)[i]);
-      igraph_vector_t *map = VECTOR(edgemaps)[i];
-      PyObject *emi = PyList_New((Py_ssize_t) no_of_edges);
-      for (j = 0; j < no_of_edges; j++) {
-        PyObject *dest = PyLong_FromLong(VECTOR(*map)[j]);
-        PyList_SET_ITEM(emi, (Py_ssize_t) j, dest);
-        /* reference to 'dest' stolen by PyList_SET_ITEM */
+      Py_ssize_t no_of_edges = igraph_ecount(VECTOR(gs)[i]);
+      igraph_vector_int_t *map = igraph_vector_int_list_get_ptr(&edgemaps, i);
+      PyObject *emi = PyList_New(no_of_edges);
+      if (emi) {
+        for (j = 0; j < no_of_edges; j++) {
+          PyObject *dest = igraphmodule_integer_t_to_PyObject(VECTOR(*map)[j]);
+          if (!dest || PyList_SetItem(emi, j, dest)) {
+            igraph_vector_ptr_destroy(&gs);
+            igraph_vector_int_list_destroy(&edgemaps);
+            Py_XDECREF(dest);
+            Py_DECREF(emi);
+            Py_DECREF(em_list);
+            return NULL;
+          }
+          /* reference to 'dest' stolen by PyList_SetItem */
+        }
       }
-      PyList_SET_ITEM(em_list, (Py_ssize_t) i, emi);
-      /* reference to 'emi' stolen by PyList_SET_ITEM */
-      igraph_vector_destroy(map);
+      if (!emi || PyList_SetItem(em_list, i, emi)) {
+        igraph_vector_ptr_destroy(&gs);
+        igraph_vector_int_list_destroy(&edgemaps);
+        Py_XDECREF(emi);
+        Py_DECREF(em_list);
+        return NULL;
+      }
+      /* reference to 'emi' stolen by PyList_SetItem */
     }
-    igraph_vector_ptr_destroy_all(&edgemaps);
-
-  } 
-  else {
+    igraph_vector_int_list_destroy(&edgemaps);
+  } else {
     /* Create intersection */
     if (igraph_intersection_many(&g, &gs, /* edgemaps */ 0)) {
       igraph_vector_ptr_destroy(&gs);
-      igraphmodule_handle_igraph_error();
-      return NULL;
+      return igraphmodule_handle_igraph_error();
     }
   }
 
@@ -299,8 +315,7 @@ PyObject *igraphmodule__intersection(PyObject *self,
     o = (igraphmodule_GraphObject*) igraphmodule_Graph_subclass_from_igraph_t(
         result_type,
         &g);
-  }
-  else {
+  } else {
     o = (igraphmodule_GraphObject*) igraphmodule_Graph_from_igraph_t(&g);
   }
 
@@ -311,8 +326,7 @@ PyObject *igraphmodule__intersection(PyObject *self,
     Py_DECREF(o);
     PyDict_SetItemString(result, "edgemaps", em_list);
     Py_DECREF(em_list);
-  }
-  else {
+  } else {
     result = (PyObject *) o;
   }
 
