@@ -40,42 +40,44 @@ The function returns one of the following values
 has_path is set to 1 if a path exists, 0 otherwise
 has_cycle is set to 1 if a cycle exists, 0 otherwise
 */
-static int igraph_i_is_eulerian_undirected(const igraph_t *graph, igraph_bool_t *has_path, igraph_bool_t *has_cycle, igraph_integer_t *start_of_path) {
+static igraph_error_t igraph_i_is_eulerian_undirected(
+        const igraph_t *graph, igraph_bool_t *has_path, igraph_bool_t *has_cycle, igraph_integer_t *start_of_path) {
     igraph_integer_t odd;
-    igraph_vector_t degree, csize;
+    igraph_vector_int_t degree;
+    igraph_vector_int_t csize;
     /* boolean vector to mark singletons: */
-    igraph_vector_t nonsingleton;
-    long int i, n, vsize;
-    long int cluster_count;
+    igraph_vector_int_t nonsingleton;
+    igraph_integer_t i, n, vsize;
+    igraph_integer_t cluster_count;
     /* number of self-looping singletons: */
-    long int es;
+    igraph_integer_t es;
     /* will be set to 1 if there are non-isolated vertices, otherwise 0: */
-    long int ens;
+    igraph_integer_t ens;
 
     n = igraph_vcount(graph);
 
     if (igraph_ecount(graph) == 0 || n <= 1) {
         start_of_path = 0; /* in case the graph has one vertex with self-loops */
-        *has_path = 1;
-        *has_cycle = 1;
+        *has_path = true;
+        *has_cycle = true;
         return  IGRAPH_SUCCESS;
     }
 
     /* check for connectedness, but singletons are special since they affect
      * the Eulerian nature only if there is a self-loop AND another edge
      * somewhere else in the graph */
-    IGRAPH_VECTOR_INIT_FINALLY(&csize, 0);
-    IGRAPH_CHECK(igraph_clusters(graph, NULL, &csize, NULL, IGRAPH_WEAK));
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&csize, 0);
+    IGRAPH_CHECK(igraph_connected_components(graph, NULL, &csize, NULL, IGRAPH_WEAK));
     cluster_count = 0;
-    vsize = igraph_vector_size(&csize);
+    vsize = igraph_vector_int_size(&csize);
     for (i = 0; i < vsize; i++) {
         if (VECTOR(csize)[i] > 1) {
             cluster_count++;
             if (cluster_count > 1) {
                 /* disconnected edges, they'll never reach each other */
-                *has_path = 0;
-                *has_cycle = 0;
-                igraph_vector_destroy(&csize);
+                *has_path = false;
+                *has_cycle = false;
+                igraph_vector_int_destroy(&csize);
                 IGRAPH_FINALLY_CLEAN(1);
 
                 return IGRAPH_SUCCESS;
@@ -83,12 +85,12 @@ static int igraph_i_is_eulerian_undirected(const igraph_t *graph, igraph_bool_t
         }
     }
 
-    igraph_vector_destroy(&csize);
+    igraph_vector_int_destroy(&csize);
     IGRAPH_FINALLY_CLEAN(1);
 
     /* the graph is connected except for singletons */
     /* find singletons (including those with self-loops) */
-    IGRAPH_VECTOR_INIT_FINALLY(&nonsingleton, 0);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&nonsingleton, 0);
     IGRAPH_CHECK(igraph_degree(graph, &nonsingleton, igraph_vss_all(), IGRAPH_ALL, IGRAPH_NO_LOOPS));
 
     /* check the degrees for odd/even:
@@ -96,13 +98,13 @@ static int igraph_i_is_eulerian_undirected(const igraph_t *graph, igraph_bool_t
      * - > 2 odd means no path
      * plus there are a few corner cases with singletons
      */
-    IGRAPH_VECTOR_INIT_FINALLY(&degree, 0);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&degree, 0);
     IGRAPH_CHECK(igraph_degree(graph, &degree, igraph_vss_all(), IGRAPH_ALL, IGRAPH_LOOPS));
     odd = 0;
     es = 0;
     ens = 0;
     for (i = 0; i < n; i++) {
-        long int deg = (long int) VECTOR(degree)[i];
+        igraph_integer_t deg = VECTOR(degree)[i];
         /* Eulerian is about edges, so skip free vertices */
         if (deg == 0) continue;
 
@@ -119,66 +121,68 @@ static int igraph_i_is_eulerian_undirected(const igraph_t *graph, igraph_bool_t
         if (es + ens > 1) {
             /* 2+ singletons with self loops or singleton with self-loops and
              * 1+ edges in the non-singleton part of the graph. */
-            *has_path = 0;
-            *has_cycle = 0;
-            igraph_vector_destroy(&nonsingleton);
-            igraph_vector_destroy(&degree);
+            *has_path = false;
+            *has_cycle = false;
+            igraph_vector_int_destroy(&nonsingleton);
+            igraph_vector_int_destroy(&degree);
             IGRAPH_FINALLY_CLEAN(2);
 
             return IGRAPH_SUCCESS;
         }
     }
 
-    igraph_vector_destroy(&nonsingleton);
+    igraph_vector_int_destroy(&nonsingleton);
     IGRAPH_FINALLY_CLEAN(1);
 
     /* this is the usual algorithm on the connected part of the graph */
     if (odd > 2) {
-        *has_path = 0;
-        *has_cycle = 0;
+        *has_path = false;
+        *has_cycle = false;
     } else if (odd == 2) {
-        *has_path = 1;
-        *has_cycle = 0;
+        *has_path = true;
+        *has_cycle = false;
     } else {
-        *has_path = 1;
-        *has_cycle = 1;
+        *has_path = true;
+        *has_cycle = true;
     }
 
     /* set start of path if there is one but there is no cycle */
     /* note: we cannot do this in the previous loop because at that time we are
      * not sure yet if a path exists */
     for (i = 0; i < n; i++) {
-        if ((*has_cycle && ((long int) VECTOR(degree)[i]) > 0) || (!*has_cycle && ((long int) VECTOR(degree)[i]) %2 == 1)) {
+        if ((*has_cycle && VECTOR(degree)[i] > 0) || (!*has_cycle && VECTOR(degree)[i] %2 == 1)) {
             *start_of_path = i;
             break;
         }
     }
 
-    igraph_vector_destroy(&degree);
+    igraph_vector_int_destroy(&degree);
     IGRAPH_FINALLY_CLEAN(1);
 
     return IGRAPH_SUCCESS;
 }
 
 
-static int igraph_i_is_eulerian_directed(const igraph_t *graph, igraph_bool_t *has_path, igraph_bool_t *has_cycle,                                         igraph_integer_t *start_of_path) {
+static igraph_error_t igraph_i_is_eulerian_directed(
+        const igraph_t *graph, igraph_bool_t *has_path, igraph_bool_t *has_cycle, igraph_integer_t *start_of_path) {
     igraph_integer_t incoming_excess, outgoing_excess, n;
-    long int i, vsize;
-    long int cluster_count;
-    igraph_vector_t out_degree, in_degree, csize;
+    igraph_integer_t i, vsize;
+    igraph_integer_t cluster_count;
+    igraph_vector_int_t out_degree, in_degree;
+    igraph_vector_int_t csize;
     /* boolean vector to mark singletons: */
-    igraph_vector_t nonsingleton;
+    igraph_vector_int_t nonsingleton;
     /* number of self-looping singletons: */
-    long int es;
+    igraph_integer_t es;
     /* will be set to 1 if there are non-isolated vertices, otherwise 0: */
-    long int ens;
+    igraph_integer_t ens;
 
     n = igraph_vcount(graph);
 
     if (igraph_ecount(graph) == 0 || n <= 1) {
         start_of_path = 0; /* in case the graph has one vertex with self-loops */
-        *has_path = 1;
-        *has_cycle = 1;
+        *has_path = true;
+        *has_cycle = true;
         return IGRAPH_SUCCESS;
     }
 
@@ -188,19 +192,19 @@ static int igraph_i_is_eulerian_directed(const igraph_t *graph, igraph_bool_t *h
     /* check for weak connectedness, but singletons are special since they affect
      * the Eulerian nature only if there is a self-loop AND another edge
      * somewhere else in the graph */
-    IGRAPH_VECTOR_INIT_FINALLY(&csize, 0);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&csize, 0);
 
-    IGRAPH_CHECK(igraph_clusters(graph, NULL, &csize, NULL, IGRAPH_WEAK));
+    IGRAPH_CHECK(igraph_connected_components(graph, NULL, &csize, NULL, IGRAPH_WEAK));
     cluster_count = 0;
-    vsize = igraph_vector_size(&csize);
+    vsize = igraph_vector_int_size(&csize);
     for (i = 0; i < vsize; i++) {
         if (VECTOR(csize)[i] > 1) {
             cluster_count++;
             if (cluster_count > 1) {
                 /* weakly disconnected edges, they'll never reach each other */
-                *has_path = 0;
-                *has_cycle = 0;
-                igraph_vector_destroy(&csize);
+                *has_path = false;
+                *has_cycle = false;
+                igraph_vector_int_destroy(&csize);
                 IGRAPH_FINALLY_CLEAN(1);
 
                 return IGRAPH_SUCCESS;
@@ -208,28 +212,28 @@ static int igraph_i_is_eulerian_directed(const igraph_t *graph, igraph_bool_t *h
         }
     }
 
-    igraph_vector_destroy(&csize);
+    igraph_vector_int_destroy(&csize);
     IGRAPH_FINALLY_CLEAN(1);
 
     /* the graph is weakly connected except for singletons */
     /* find the singletons (including those with self-loops) */
-    IGRAPH_VECTOR_INIT_FINALLY(&nonsingleton, 0);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&nonsingleton, 0);
     IGRAPH_CHECK(igraph_degree(graph, &nonsingleton, igraph_vss_all(), IGRAPH_ALL, IGRAPH_NO_LOOPS));
 
 
     /* checking if no. of incoming edges == outgoing edges
      * plus there are a few corner cases with singletons */
-    IGRAPH_VECTOR_INIT_FINALLY(&out_degree, 0);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&out_degree, 0);
     IGRAPH_CHECK(igraph_degree(graph, &out_degree, igraph_vss_all(), IGRAPH_OUT, IGRAPH_LOOPS));
 
-    IGRAPH_VECTOR_INIT_FINALLY(&in_degree, 0);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&in_degree, 0);
     IGRAPH_CHECK(igraph_degree(graph, &in_degree, igraph_vss_all(), IGRAPH_IN, IGRAPH_LOOPS));
     es = 0;
     ens = 0;
     *start_of_path = -1;
     for (i = 0; i < n; i++) {
-        long int degin = VECTOR(in_degree)[i];
-        long int degout = VECTOR(out_degree)[i];
+        igraph_integer_t degin = VECTOR(in_degree)[i];
+        igraph_integer_t degout = VECTOR(out_degree)[i];
 
         /* Eulerian is about edges, so skip free vertices */
         if (degin + degout == 0) continue;
@@ -247,11 +251,11 @@ static int igraph_i_is_eulerian_directed(const igraph_t *graph, igraph_bool_t *h
         if (es + ens > 1) {
             /* 2+ singletons with self loops or singleton with self-loops and
              * 1+ edges in the non-singleton part of the graph. */
-            *has_path = 0;
-            *has_cycle = 0;
-            igraph_vector_destroy(&nonsingleton);
-            igraph_vector_destroy(&in_degree);
-            igraph_vector_destroy(&out_degree);
+            *has_path = false;
+            *has_cycle = false;
+            igraph_vector_int_destroy(&nonsingleton);
+            igraph_vector_int_destroy(&in_degree);
+            igraph_vector_int_destroy(&out_degree);
             IGRAPH_FINALLY_CLEAN(3);
 
             return IGRAPH_SUCCESS;
@@ -282,25 +286,25 @@ static int igraph_i_is_eulerian_directed(const igraph_t *graph, igraph_bool_t *h
          * 1. 1+ vertices have 2+ in/out
          * 2. 2+ nodes have 1+ in/out */
         if (incoming_excess > 1 || outgoing_excess > 1) {
-            *has_path = 0;
-            *has_cycle = 0;
-            igraph_vector_destroy(&nonsingleton);
-            igraph_vector_destroy(&in_degree);
-            igraph_vector_destroy(&out_degree);
+            *has_path = false;
+            *has_cycle = false;
+            igraph_vector_int_destroy(&nonsingleton);
+            igraph_vector_int_destroy(&in_degree);
+            igraph_vector_int_destroy(&out_degree);
             IGRAPH_FINALLY_CLEAN(3);
 
             return IGRAPH_SUCCESS;
         }
     }
 
-    *has_path = 1;
+    *has_path = true;
     /* perfect edge balance -> strong connectivity */
     *has_cycle = (incoming_excess == 0) && (outgoing_excess == 0);
     /* either way, the start was set already */
 
-    igraph_vector_destroy(&nonsingleton);
-    igraph_vector_destroy(&in_degree);
-    igraph_vector_destroy(&out_degree);
+    igraph_vector_int_destroy(&nonsingleton);
+    igraph_vector_int_destroy(&in_degree);
+    igraph_vector_int_destroy(&out_degree);
     IGRAPH_FINALLY_CLEAN(3);
 
     return IGRAPH_SUCCESS;
@@ -310,7 +314,7 @@ static int igraph_i_is_eulerian_directed(const igraph_t *graph, igraph_bool_t *h
 /**
  * \ingroup Eulerian
  * \function igraph_is_eulerian
- * \brief Checks whether an Eulerian path or cycle exists
+ * \brief Checks whether an Eulerian path or cycle exists.
  *
  * An Eulerian path traverses each edge of the graph precisely once. A closed
  * Eulerian path is referred to as an Eulerian cycle.
@@ -326,7 +330,7 @@ static int igraph_i_is_eulerian_directed(const igraph_t *graph, igraph_bool_t *h
  *
  */
 
-int igraph_is_eulerian(const igraph_t *graph, igraph_bool_t *has_path, igraph_bool_t *has_cycle) {
+igraph_error_t igraph_is_eulerian(const igraph_t *graph, igraph_bool_t *has_path, igraph_bool_t *has_cycle) {
     igraph_integer_t start_of_path = 0;
 
     if (igraph_is_directed(graph)) {
@@ -338,66 +342,70 @@ int igraph_is_eulerian(const igraph_t *graph, igraph_bool_t *has_path, igraph_bo
 }
 
 
-static int igraph_i_eulerian_path_undirected(const igraph_t *graph, igraph_vector_t *edge_res, igraph_vector_t *vertex_res, igraph_integer_t start_of_path) {
-    long int curr;
+static igraph_error_t igraph_i_eulerian_path_undirected(
+        const igraph_t *graph, igraph_vector_int_t *edge_res, igraph_vector_int_t *vertex_res,
+        igraph_integer_t start_of_path) {
+
+    igraph_integer_t curr;
     igraph_integer_t n, m;
     igraph_inclist_t il;
-    igraph_stack_t path, tracker, edge_tracker, edge_path;
+    igraph_stack_int_t path, tracker, edge_tracker, edge_path;
     igraph_vector_bool_t visited_list;
-    igraph_vector_t degree;
+    igraph_vector_int_t degree;
 
     n = igraph_vcount(graph);
     m = igraph_ecount(graph);
 
     if (edge_res) {
-        igraph_vector_clear(edge_res);
+        igraph_vector_int_clear(edge_res);
     }
 
     if (vertex_res) {
-        igraph_vector_clear(vertex_res);
+        igraph_vector_int_clear(vertex_res);
     }
 
     if (m == 0 || n == 0) {
         return IGRAPH_SUCCESS;
     }
 
-    IGRAPH_VECTOR_INIT_FINALLY(&degree, 0);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&degree, 0);
     IGRAPH_CHECK(igraph_degree(graph, &degree, igraph_vss_all(), IGRAPH_ALL, IGRAPH_LOOPS));
 
-    IGRAPH_CHECK(igraph_stack_init(&path, n));
-    IGRAPH_FINALLY(igraph_stack_destroy, &path);
+    IGRAPH_CHECK(igraph_stack_int_init(&path, n));
+    IGRAPH_FINALLY(igraph_stack_int_destroy, &path);
 
-    IGRAPH_CHECK(igraph_stack_init(&tracker, n));
-    IGRAPH_FINALLY(igraph_stack_destroy, &tracker);
+    IGRAPH_CHECK(igraph_stack_int_init(&tracker, n));
+    IGRAPH_FINALLY(igraph_stack_int_destroy, &tracker);
 
-    IGRAPH_CHECK(igraph_stack_init(&edge_path, n));
-    IGRAPH_FINALLY(igraph_stack_destroy, &edge_path);
+    IGRAPH_CHECK(igraph_stack_int_init(&edge_path, n));
+    IGRAPH_FINALLY(igraph_stack_int_destroy, &edge_path);
 
-    IGRAPH_CHECK(igraph_stack_init(&edge_tracker, n));
-    IGRAPH_FINALLY(igraph_stack_destroy, &edge_tracker);
+    IGRAPH_CHECK(igraph_stack_int_init(&edge_tracker, n));
+    IGRAPH_FINALLY(igraph_stack_int_destroy, &edge_tracker);
 
     IGRAPH_VECTOR_BOOL_INIT_FINALLY(&visited_list, m);
 
-    IGRAPH_CHECK(igraph_stack_push(&tracker, start_of_path));
+    IGRAPH_CHECK(igraph_stack_int_push(&tracker, start_of_path));
 
     IGRAPH_CHECK(igraph_inclist_init(graph, &il, IGRAPH_OUT, IGRAPH_LOOPS_ONCE));
     IGRAPH_FINALLY(igraph_inclist_destroy, &il);
 
     curr = start_of_path;
 
-    while (!igraph_stack_empty(&tracker)) {
+    while (!igraph_stack_int_empty(&tracker)) {
 
         if (VECTOR(degree)[curr] != 0) {
             igraph_vector_int_t *incedges;
-            long nc, edge;
-            long int j, next;
-            IGRAPH_CHECK(igraph_stack_push(&tracker, curr));
+            igraph_integer_t nc, edge = -1;
+            igraph_integer_t j, next;
+            IGRAPH_CHECK(igraph_stack_int_push(&tracker, curr));
 
             incedges = igraph_inclist_get(&il, curr);
             nc = igraph_vector_int_size(incedges);
+            IGRAPH_ASSERT(nc > 0);
 
             for (j = 0; j < nc; j++) {
-                edge = (long) VECTOR(*incedges)[j];
+                edge = VECTOR(*incedges)[j];
                 if (!VECTOR(visited_list)[edge]) {
                     break;
                 }
@@ -405,7 +413,7 @@ static int igraph_i_eulerian_path_undirected(const igraph_t *graph, igraph_vecto
 
             next = IGRAPH_OTHER(graph, edge, curr);
 
-            IGRAPH_CHECK(igraph_stack_push(&edge_tracker, edge));
+            IGRAPH_CHECK(igraph_stack_int_push(&edge_tracker, edge));
 
             /* remove edge here */
             VECTOR(degree)[curr]--;
@@ -415,101 +423,105 @@ static int igraph_i_eulerian_path_undirected(const igraph_t *graph, igraph_vecto
             curr = next;
         } else { /* back track to find remaining circuit */
             igraph_integer_t curr_e;
-            IGRAPH_CHECK(igraph_stack_push(&path, curr));
-            curr = igraph_stack_pop(&tracker);
-            if (!igraph_stack_empty(&edge_tracker)) {
-                curr_e = igraph_stack_pop(&edge_tracker);
-                IGRAPH_CHECK(igraph_stack_push(&edge_path, curr_e));
+            IGRAPH_CHECK(igraph_stack_int_push(&path, curr));
+            curr = igraph_stack_int_pop(&tracker);
+            if (!igraph_stack_int_empty(&edge_tracker)) {
+                curr_e = igraph_stack_int_pop(&edge_tracker);
+                IGRAPH_CHECK(igraph_stack_int_push(&edge_path, curr_e));
             }
         }
     }
 
     if (edge_res) {
-        IGRAPH_CHECK(igraph_vector_reserve(edge_res, m));
-        while (!igraph_stack_empty(&edge_path)) {
-            IGRAPH_CHECK(igraph_vector_push_back(edge_res, igraph_stack_pop(&edge_path)));
+        IGRAPH_CHECK(igraph_vector_int_reserve(edge_res, m));
+        while (!igraph_stack_int_empty(&edge_path)) {
+            IGRAPH_CHECK(igraph_vector_int_push_back(edge_res, igraph_stack_int_pop(&edge_path)));
         }
     }
     if (vertex_res) {
-        IGRAPH_CHECK(igraph_vector_reserve(vertex_res, m+1));
-        while (!igraph_stack_empty(&path)) {
-            IGRAPH_CHECK(igraph_vector_push_back(vertex_res, igraph_stack_pop(&path)));
+        IGRAPH_CHECK(igraph_vector_int_reserve(vertex_res, m+1));
+        while (!igraph_stack_int_empty(&path)) {
+            IGRAPH_CHECK(igraph_vector_int_push_back(vertex_res, igraph_stack_int_pop(&path)));
         }
     }
 
-    igraph_stack_destroy(&path);
-    igraph_stack_destroy(&tracker);
-    igraph_stack_destroy(&edge_path);
-    igraph_stack_destroy(&edge_tracker);
+    igraph_stack_int_destroy(&path);
+    igraph_stack_int_destroy(&tracker);
+    igraph_stack_int_destroy(&edge_path);
+    igraph_stack_int_destroy(&edge_tracker);
     igraph_vector_bool_destroy(&visited_list);
     igraph_inclist_destroy(&il);
-    igraph_vector_destroy(&degree);
+    igraph_vector_int_destroy(&degree);
     IGRAPH_FINALLY_CLEAN(7);
 
     return IGRAPH_SUCCESS;
 }
 
 /* solution adapted from https://www.geeksforgeeks.org/hierholzers-algorithm-directed-graph/ */
-static int igraph_i_eulerian_path_directed(const igraph_t *graph, igraph_vector_t *edge_res, igraph_vector_t *vertex_res, igraph_integer_t start_of_path) {
-    long int curr;
+static igraph_error_t igraph_i_eulerian_path_directed(
+        const igraph_t *graph, igraph_vector_int_t *edge_res, igraph_vector_int_t *vertex_res,
+        igraph_integer_t start_of_path) {
+
+    igraph_integer_t curr;
     igraph_integer_t n, m;
     igraph_inclist_t il;
-    igraph_stack_t path, tracker, edge_tracker, edge_path;
+    igraph_stack_int_t path, tracker, edge_tracker, edge_path;
     igraph_vector_bool_t visited_list;
-    igraph_vector_t remaining_out_edges;
+    igraph_vector_int_t remaining_out_edges;
 
     n = igraph_vcount(graph);
     m = igraph_ecount(graph);
 
     if (edge_res) {
-        igraph_vector_clear(edge_res);
+        igraph_vector_int_clear(edge_res);
     }
 
     if (vertex_res) {
-        igraph_vector_clear(vertex_res);
+        igraph_vector_int_clear(vertex_res);
     }
 
     if (m == 0 || n == 0) {
         return IGRAPH_SUCCESS;
     }
 
-    IGRAPH_CHECK(igraph_stack_init(&path, n));
-    IGRAPH_FINALLY(igraph_stack_destroy, &path);
+    IGRAPH_CHECK(igraph_stack_int_init(&path, n));
+    IGRAPH_FINALLY(igraph_stack_int_destroy, &path);
 
-    IGRAPH_CHECK(igraph_stack_init(&tracker, n));
-    IGRAPH_FINALLY(igraph_stack_destroy, &tracker);
+    IGRAPH_CHECK(igraph_stack_int_init(&tracker, n));
+    IGRAPH_FINALLY(igraph_stack_int_destroy, &tracker);
 
-    IGRAPH_CHECK(igraph_stack_init(&edge_path, n));
-    IGRAPH_FINALLY(igraph_stack_destroy, &edge_path);
+    IGRAPH_CHECK(igraph_stack_int_init(&edge_path, n));
+    IGRAPH_FINALLY(igraph_stack_int_destroy, &edge_path);
 
-    IGRAPH_CHECK(igraph_stack_init(&edge_tracker, n));
-    IGRAPH_FINALLY(igraph_stack_destroy, &edge_tracker);
+    IGRAPH_CHECK(igraph_stack_int_init(&edge_tracker, n));
+    IGRAPH_FINALLY(igraph_stack_int_destroy, &edge_tracker);
 
     IGRAPH_VECTOR_BOOL_INIT_FINALLY(&visited_list, m);
 
-    IGRAPH_CHECK(igraph_stack_push(&tracker, start_of_path));
+    IGRAPH_CHECK(igraph_stack_int_push(&tracker, start_of_path));
 
     IGRAPH_CHECK(igraph_inclist_init(graph, &il, IGRAPH_OUT, IGRAPH_LOOPS_ONCE));
     IGRAPH_FINALLY(igraph_inclist_destroy, &il);
 
-    IGRAPH_VECTOR_INIT_FINALLY(&remaining_out_edges, 0);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&remaining_out_edges, 0);
     IGRAPH_CHECK(igraph_degree(graph, &remaining_out_edges, igraph_vss_all(), IGRAPH_OUT, IGRAPH_LOOPS));
 
     curr = start_of_path;
 
-    while (!igraph_stack_empty(&tracker)) {
+    while (!igraph_stack_int_empty(&tracker)) {
 
         if (VECTOR(remaining_out_edges)[curr] != 0) {
             igraph_vector_int_t *incedges;
-            long nc, edge;
-            long int j, next;
-            IGRAPH_CHECK(igraph_stack_push(&tracker, curr));
+            igraph_integer_t nc, edge = -1;
+            igraph_integer_t j, next;
+            IGRAPH_CHECK(igraph_stack_int_push(&tracker, curr));
 
             incedges = igraph_inclist_get(&il, curr);
             nc = igraph_vector_int_size(incedges);
+            IGRAPH_ASSERT(nc > 0);
 
             for (j = 0; j < nc; j++) {
-                edge = (long) VECTOR(*incedges)[j];
+                edge = VECTOR(*incedges)[j];
                 if (!VECTOR(visited_list)[edge]) {
                     break;
                 }
@@ -517,7 +529,7 @@ static int igraph_i_eulerian_path_directed(const igraph_t *graph, igraph_vector_
 
             next = IGRAPH_TO(graph, edge);
 
-            IGRAPH_CHECK(igraph_stack_push(&edge_tracker, edge));
+            IGRAPH_CHECK(igraph_stack_int_push(&edge_tracker, edge));
 
             /* remove edge here */
             VECTOR(remaining_out_edges)[curr]--;
@@ -526,35 +538,35 @@ static int igraph_i_eulerian_path_directed(const igraph_t *graph, igraph_vector_
             curr = next;
         } else { /* back track to find remaining circuit */
             igraph_integer_t curr_e;
-            IGRAPH_CHECK(igraph_stack_push(&path, curr));
-            curr = igraph_stack_pop(&tracker);
-            if (!igraph_stack_empty(&edge_tracker)) {
-                curr_e = igraph_stack_pop(&edge_tracker);
-                IGRAPH_CHECK(igraph_stack_push(&edge_path, curr_e));
+            IGRAPH_CHECK(igraph_stack_int_push(&path, curr));
+            curr = igraph_stack_int_pop(&tracker);
+            if (!igraph_stack_int_empty(&edge_tracker)) {
+                curr_e = igraph_stack_int_pop(&edge_tracker);
+                IGRAPH_CHECK(igraph_stack_int_push(&edge_path, curr_e));
             }
         }
     }
 
     if (edge_res) {
-        IGRAPH_CHECK(igraph_vector_reserve(edge_res, m));
-        while (!igraph_stack_empty(&edge_path)) {
-            IGRAPH_CHECK(igraph_vector_push_back(edge_res, igraph_stack_pop(&edge_path)));
+        IGRAPH_CHECK(igraph_vector_int_reserve(edge_res, m));
+        while (!igraph_stack_int_empty(&edge_path)) {
+            IGRAPH_CHECK(igraph_vector_int_push_back(edge_res, igraph_stack_int_pop(&edge_path)));
         }
     }
     if (vertex_res) {
-        IGRAPH_CHECK(igraph_vector_reserve(vertex_res, m+1));
-        while (!igraph_stack_empty(&path)) {
-            IGRAPH_CHECK(igraph_vector_push_back(vertex_res, igraph_stack_pop(&path)));
+        IGRAPH_CHECK(igraph_vector_int_reserve(vertex_res, m+1));
+        while (!igraph_stack_int_empty(&path)) {
+            IGRAPH_CHECK(igraph_vector_int_push_back(vertex_res, igraph_stack_int_pop(&path)));
         }
     }
 
-    igraph_stack_destroy(&path);
-    igraph_stack_destroy(&tracker);
-    igraph_stack_destroy(&edge_path);
-    igraph_stack_destroy(&edge_tracker);
+    igraph_stack_int_destroy(&path);
+    igraph_stack_int_destroy(&tracker);
+    igraph_stack_int_destroy(&edge_path);
+    igraph_stack_int_destroy(&edge_tracker);
     igraph_vector_bool_destroy(&visited_list);
     igraph_inclist_destroy(&il);
-    igraph_vector_destroy(&remaining_out_edges);
+    igraph_vector_int_destroy(&remaining_out_edges);
     IGRAPH_FINALLY_CLEAN(7);
 
     return IGRAPH_SUCCESS;
@@ -564,7 +576,7 @@ static int igraph_i_eulerian_path_directed(const igraph_t *graph, igraph_vector_
 /**
  * \ingroup Eulerian
  * \function igraph_eulerian_cycle
- * \brief Finds an Eulerian cycle
+ * \brief Finds an Eulerian cycle.
  *
  * Finds an Eulerian cycle, if it exists. An Eulerian cycle is a closed path
  * that traverses each edge precisely once.
@@ -591,7 +603,9 @@ static int igraph_i_eulerian_path_directed(const igraph_t *graph, igraph_vector_
  *
  */
 
-int igraph_eulerian_cycle(const igraph_t *graph, igraph_vector_t *edge_res, igraph_vector_t *vertex_res) {
+igraph_error_t igraph_eulerian_cycle(
+        const igraph_t *graph, igraph_vector_int_t *edge_res, igraph_vector_int_t *vertex_res) {
+
     igraph_bool_t has_cycle;
     igraph_bool_t has_path;
     igraph_integer_t start_of_path = 0;
@@ -621,7 +635,7 @@ int igraph_eulerian_cycle(const igraph_t *graph, igraph_vector_t *edge_res, igra
 /**
  * \ingroup Eulerian
  * \function igraph_eulerian_path
- * \brief Finds an Eulerian path
+ * \brief Finds an Eulerian path.
  *
  * Finds an Eulerian path, if it exists. An Eulerian path traverses
  * each edge precisely once.
@@ -648,7 +662,9 @@ int igraph_eulerian_cycle(const igraph_t *graph, igraph_vector_t *edge_res, igra
  *
  */
 
-int igraph_eulerian_path(const igraph_t *graph, igraph_vector_t *edge_res, igraph_vector_t *vertex_res) {
+igraph_error_t igraph_eulerian_path(
+        const igraph_t *graph, igraph_vector_int_t *edge_res, igraph_vector_int_t *vertex_res) {
+
     igraph_bool_t has_cycle;
     igraph_bool_t has_path;
     igraph_integer_t start_of_path = 0;
